@@ -2,6 +2,7 @@ library("leiden")
 library("reticulate")
 library("igraph")
 context("running Leiden on unweighted objects")
+set.seed(9000)
 
 modules <- reticulate::py_module_available("leidenalg") && reticulate::py_module_available("igraph")
 
@@ -21,39 +22,54 @@ graph <- igraph::graph_from_adjacency_matrix(mat, weighted = NULL)
 test_that("run with unweighted dense matrix", {
   skip_if_no_python()
   set.seed(9000)
-  mat <- round(matrix(runif(100, max = 1), 10, 10, ), 0)
+  mat <- round(matrix(runif(100, max = 1), 10, 10), 0)
   mat
-  part_mat0 <- leiden::leiden(mat, seed = 9000)
-  expect_equal(part_mat0, c(1, 2, 2, 2, 1, 2, 2, 1, 1, 1))
+  part_mat0 <- leiden::leiden(mat, seed = 9000, resolution_parameter = 0.8)
+  expect_equal(table(part_mat0), structure(c(`1` = 5L, `2` = 5L), .Dim = 2L, .Dimnames = list(
+    part_mat0 = c("1", "2")), class = "table"))
+  expect_length(object = unique(part_mat0[which(c(1, 2, 2, 2, 1, 2, 2, 1, 1, 1) == 1)]), n = 1)
+  expect_length(object = unique(part_mat0[which(c(1, 2, 2, 2, 1, 2, 2, 1, 1, 1) == 2)]), n = 1)
 })
 
+set.seed(9000)
 test_that("run with unweighted sparse matrix", {
   skip_if_no_python()
+  set.seed(9000)
   sp.mat <- as(mat, Class = "dgCMatrix")
   part_sp.mat0 <- leiden::leiden(sp.mat, seed = 9000)
-  expect_equal(part_sp.mat0, c(1, 2, 2, 2, 1, 2, 2, 1, 1, 1))
+  expect_equal(table(part_sp.mat0), structure(c(`1` = 5L, `2` = 5L), .Dim = 2L, .Dimnames = list(
+    part_sp.mat0 = c("1", "2")), class = "table"))
+  expect_length(object = unique(part_sp.mat0[which(c(1, 2, 2, 2, 1, 2, 2, 1, 1, 1) == 1)]), n = 1)
+  expect_length(object = unique(part_sp.mat0[which(c(1, 2, 2, 2, 1, 2, 2, 1, 1, 1) == 2)]), n = 1)
 })
 
-
+set.seed(9000)
 test_that("run with unweighted graph object", {
   skip_if_no_python()
+  set.seed(9000)
   graph <- igraph::graph_from_adjacency_matrix(mat, weighted = NULL)
   graph
   part_graph0 <- leiden::leiden(graph, seed = 9000)
   part_graph0
-  expect_equal(part_graph0,  c(1, 2, 2, 2, 1, 2, 2, 1, 1, 1))
+  expect_equal(table(part_graph0), structure(c(`1` = 5L, `2` = 5L), .Dim = 2L, .Dimnames = list(
+    part_graph0 = c("1", "2")), class = "table"))
+  expect_length(object = unique(part_graph0[which(c(1, 2, 2, 2, 1, 2, 2, 1, 1, 1) == 1)]), n = 1)
+  expect_length(object = unique(part_graph0[which(c(1, 2, 2, 2, 1, 2, 2, 1, 1, 1) == 2)]), n = 1)
+
 })
 
+set.seed(9000)
 test_that("same output with different input class", {
   skip_if_no_python()
-  part_mat0 <- leiden::leiden(mat, seed = 9000)
-  part_graph0 <- leiden::leiden(graph, seed = 9000)
-  expect_equivalent(part_mat0, c(1, 2, 2, 2, 1, 2, 2, 1, 1, 1))
-  expect_equivalent(part_graph0, c(1, 2, 2, 2, 1, 2, 2, 1, 1, 1))
-  expect_equivalent(part_mat0, part_graph0)
-  expect_equal(sum(diag(table(part_mat0, part_graph0))), length(part_mat0))
-  expect_equal(sum(diag(table(part_mat0, part_graph0))), nrow(mat))
-  expect_equal(sum(diag(table(part_mat0, part_graph0))), length(V(graph)))
+  set.seed(9000)
+  part_mat0 <- leiden::leiden(mat, seed = 9000, resolution_parameter = 0.8)
+  set.seed(9000)
+  part_graph0 <- leiden::leiden(graph, seed = 9000, resolution_parameter = 0.8)
+  expect_length(object = unique(part_mat0[which(c(1, 2, 2, 2, 1, 2, 2, 1, 1, 1) == 1)]), n = 1)
+  expect_length(object = unique(part_mat0[which(c(1, 2, 2, 2, 1, 2, 2, 1, 1, 1) == 2)]), n = 1)
+  expect_length(object = unique(part_graph0[which(c(1, 2, 2, 2, 1, 2, 2, 1, 1, 1) == 1)]), n = 1)
+  expect_length(object = unique(part_graph0[which(c(1, 2, 2, 2, 1, 2, 2, 1, 1, 1) == 2)]), n = 1)
+  expect_true(all(part_mat0 + part_graph0 == 3) ||  all(part_mat0 == part_graph0))
 })
 
 # # Test Seurat (development version)
@@ -102,6 +118,7 @@ test_that("run with weighted dense matrix", {
 
 test_that("run with weighted sparse matrix", {
   skip_if_no_python()
+  set.seed(9000)
   sp.mat <- as(mat, Class = "dgCMatrix")
   #passing weights
   part_sp.mat1 <- leiden::leiden(sp.mat, seed = 9000, weights = mat[mat != 0])
@@ -114,6 +131,7 @@ test_that("run with weighted sparse matrix", {
 
 test_that("run with weighted graph object", {
   skip_if_no_python()
+  set.seed(9000)
   graph <- igraph::graph_from_adjacency_matrix(mat, weighted = NULL)
   graph
   #passing weights
@@ -126,6 +144,7 @@ test_that("run with weighted graph object", {
 
 test_that("same output with different input class", {
   skip_if_no_python()
+  set.seed(9000)
   part_mat1 <- leiden::leiden(mat, seed = 9000)
   part_graph1 <- leiden::leiden(graph, seed = 9000)
   expect_equivalent(part_mat1, c(2, 1, 1, 1, 2, 1, 1, 2, 2, 2))
@@ -158,4 +177,3 @@ test_that("same output with different input class", {
 # sum(diag(table(part_mat2, part_graph2))) == length(part_mat)
 # sum(diag(table(part_mat2, part_graph2))) == nrow(mat)
 # sum(diag(table(part_mat2, part_graph2))) == length(V(graph))
-
